# coding=utf-8

import requests
from lxml import etree

headers = {
    'Connection': 'keep-alive',
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_1_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.96 Safari/537.36',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'Sec-Fetch-Site': 'same-origin',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-User': '?1',
    'Sec-Fetch-Dest': 'document',
    'Referer': 'https://www.baidu.com/',
    'Accept-Language': 'zh-CN,zh;q=0.9',
}


class BaiduSearch(object):
    def __init__(self, keyword, cookies, domain=None):
        self.keyword = keyword
        self.domain = domain
        self.cookies = cookies

    def search(self, keyword, pn):
        params = (
            ('wd', keyword),
            ('pn', (pn - 1) * 10),
            ('oq', keyword),
            ('tn', 'baiduhome_pg'),
            ('ie', 'utf-8'),
            ('rsv_idx', '2'),
            ('rsv_pq', 'd09ea91a000533ad'),
            ('rsv_t', 'a741enhrt8jcViHd/8Q+gb0DnCzjIbctyKmpOkRk6BibYwnyQXvHFSqrZtTKeUHQlE4s'),
        )
        headers['Cookie'] = self.cookies
        response = requests.get('https://www.baidu.com/s', headers=headers, params=params)
        return response

    @classmethod
    def get_url(cls, url):
        """extract special url"""
        try:
            r = requests.get(url, allow_redirects=False)
            if r.status_code == 302 and 'Location' in r.headers.keys():
                return r.headers['Location']
        except Exception:
            pass
        return ''

    def get_search(self):
        keyword = 'site: {} {}'.format(self.domain, self.keyword) if self.domain else self.keyword
        for pn in range(1, 76):
            response = self.search(keyword, pn)
            html = etree.HTML(response.text)
            items = html.xpath('//*/h3/a')
            for item in items:
                title = ''.join(item.xpath('.//text()'))
                href = item.xpath('./@href')[0]
                url = self.get_url(href)
                if not url:
                    continue
                if self.domain and self.domain not in url:
                    continue
                yield {'title': title, 'url': url}


if __name__ == '__main__':
    _keyword = '大碗面'
    _domain = 'dianping.com'
    _cookies = '这里放cookie'
    b = BaiduSearch(_keyword, _cookies, _domain)
    for item in b.get_search():
        print(item)



import queue
import math
import datetime
import random
import time
import json

import requests


headers = {
    'Host': 'index.baidu.com',
    'Connection': 'keep-alive',
    'X-Requested-With': 'XMLHttpRequest',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
}


def get_time_range_list(startdate, enddate):
    """
        切分时间段
    """
    date_range_list = []
    startdate = datetime.datetime.strptime(startdate, '%Y-%m-%d')
    enddate = datetime.datetime.strptime(enddate, '%Y-%m-%d')
    while 1:
        tempdate = startdate + datetime.timedelta(days=300)
        if tempdate > enddate:
            date_range_list.append((startdate, enddate))
            break
        date_range_list.append((startdate, tempdate))
        startdate = tempdate + datetime.timedelta(days=1)
    return date_range_list


def split_keywords(keywords: list) -> [list]:
    """
    一个请求最多传入5个关键词, 所以需要对关键词进行切分
    """
    return [keywords[i*5: (i+1)*5] for i in range(math.ceil(len(keywords)/5))]


def get_params_queue(start_date, end_date, keywords):
    """
        获取参数队列
    """
    params_queue = queue.Queue()
    keywords_list = split_keywords(keywords)
    time_range_list = get_time_range_list(start_date, end_date)
    for start_date, end_date in time_range_list:
        for keywords in keywords_list:
            params = {
                'keywords': keywords,
                'start_date': start_date,
                'end_date': end_date
            }
            params_queue.put(params)
    return params_queue


def http_get(url, cookies):
    """
        发送get请求, 程序中所有的get都是调这个方法
        如果想使用多cookies抓取, 和请求重试功能
        在这自己添加
    """
    _headers = headers.copy()
    _headers['Cookie'] = cookies
    response = requests.get(url, headers=_headers, timeout=5)
    if response.status_code != 200:
        raise requests.Timeout
    return response.text


def get_key(uniqid, cookies):
    """
    """
    url = 'http://index.baidu.com/Interface/api/ptbk?uniqid=%s' % uniqid
    html = http_get(url, cookies)
    datas = json.loads(html)
    key = datas['data']
    return key


def decrypt_func(key, data):
    """
        数据解密方法
    """
    a = key
    i = data
    n = {}
    s = []
    for o in range(len(a)//2):
        n[a[o]] = a[len(a)//2 + o]
    for r in range(len(data)):
        s.append(n[i[r]])
    return ''.join(s).split(',')


def sleep_func():
    """
        sleep方法, 单账号抓取过快, 一段时间内请求会失败
    """
    sleep_time = random.choice(range(50, 90)) * 0.1
    time.sleep(sleep_time)


def test_cookies(cookies):
    """
        测试cookie是否可用
    """
    html = http_get('https://www.baidu.com/', cookies)
    return '退出' in html



from urllib.parse import urlencode
import queue
import datetime
import json

import requests

from . import utils


class ExtendedBaiduIndex:
    """
        百度搜索指数(获取咨询指数或者媒体指数)
        :keywords; list
        :start_date; string '2018-10-02'
        :end_date; string '2018-10-02'
        :type; string 'news'(媒体) or 'feed'(咨询)
        :area; int, search by cls.province_code/cls.city_code
    """
    pre_url_dict = {
        'news': 'http://index.baidu.com/api/NewsApi/getNewsIndex?',
        'feed': 'http://index.baidu.com/api/FeedSearchApi/getFeedIndex?'
    }

    def __init__(
        self,
        *, 
        keywords: list,
        start_date: str,
        end_date: str,
        cookies: str,
        kind="news",
        area=0
    ):
        self.kind = kind
        self.keywords = keywords
        self.area = area
        self.start_date = start_date
        self.end_date = end_date
        self.cookies = cookies
        self._pre_url = self.pre_url_dict[kind]
        self._params_queue = utils.get_params_queue(start_date, end_date, keywords)

    def get_index(self):
        """
        获取百度指数
        返回的数据格式为:
        {
            'keyword': '武林外传',
            'type': 'wise',
            'date': '2019-04-30',
            'index': '202'
        }
        """
        while 1:
            try:
                params_data = self._params_queue.get(timeout=1)
                encrypt_datas, uniqid = self._get_encrypt_datas(
                    start_date=params_data['start_date'],
                    end_date=params_data['end_date'],
                    keywords=params_data['keywords']
                )
                key = utils.get_key(uniqid, self.cookies)
                for encrypt_data in encrypt_datas:
                    encrypt_data['data'] = utils.decrypt_func(key, encrypt_data['data'])
                    for formated_data in self._format_data(encrypt_data):
                        yield formated_data
            except requests.Timeout:
                self._params_queue.put(params_data)
            except queue.Empty:
                break
            utils.sleep_func()

    def _get_encrypt_datas(self, start_date, end_date, keywords):
        """
        :start_date; str, 2018-10-01
        :end_date; str, 2018-10-01
        :keyword; list, ['1', '2', '3']
        """
        word_list = [
            [{'name': keyword, 'wordType': 1} for keyword in keyword_list]
            for keyword_list in keywords
        ]
        request_args = {
            'word': json.dumps(word_list),
            'startDate': start_date.strftime('%Y-%m-%d'),
            'endDate': end_date.strftime('%Y-%m-%d'),
            'area': self.area
        }
        url = self._pre_url + urlencode(request_args)
        html = utils.http_get(url, self.cookies)
        datas = json.loads(html)
        uniqid = datas['data']['uniqid']
        encrypt_datas = []
        for single_data in datas['data']['index']:
            encrypt_datas.append(single_data)
        return (encrypt_datas, uniqid)

    def _format_data(self, data):
        """
            格式化堆在一起的数据
        """
        keyword = str(data['key'])
        start_date = datetime.datetime.strptime(data['startDate'], '%Y-%m-%d')
        end_date = datetime.datetime.strptime(data['endDate'], '%Y-%m-%d')
        date_list = []
        while start_date <= end_date:
            date_list.append(start_date)
            start_date += datetime.timedelta(days=1)

        index_datas = data['data']
        for i, cur_date in enumerate(date_list):
            try:
                index_data = index_datas[i]
            except IndexError:
                index_data = ''
            formated_data = {
                'keyword': [keyword_info['name'] for keyword_info in json.loads(keyword.replace('\'', '"'))],
                'date': cur_date.strftime('%Y-%m-%d'),
                'index': index_data if index_data else '0'
            }
            yield formated_data
